# Displacement sensitivity calculation for KAGRA auxiliary length signals
# Based on kagra_sensitivity.py
# Initiated by Yuta Michimura and Kentaro Komori (2020.06.30)

from math import *
import numpy as np
import matplotlib.pyplot as plt
import scipy.integrate as integ
i=1j

#####physical constants and other basic parameters
#from Rev. Mod. Phys. 88, 035009 (2016)
c=2.99792458e8		#speed of light (m/s)
hbar=1.054571800e-34  #Dirac constant (J*s)
kb=1.38064852e-23	 #Boltzmann constant (J/K)
e=1.6021766208e-19	#electric charge (C)
G=6.67384e-11		 #gravitational constant (m**3/kg/s**2)
g=9.80665			 #gravitational acceleration (m/s**2)
lamb=1.064e-6		 #laser wave length (m)
omegal=2*pi*c/lamb	#laser frequency (rad/s)
L=3e3	 #arm length (m)

fres=1000			   #frequency resolution
f=np.logspace(0,4,fres)
omega=2*pi*f

##### FOR EACH OPTIC OR DOF
def seismicnoise(optic):
	if "ITM" in optic or "ETM" in optic:
		xseis=np.sqrt((2.5e-13*f**(-6.8))**2+(1.1e-12*f**(-8))**2+(1.1e-14*f**(-5.7))**2)
		xggn=1.2e-16/f**4/2
		return np.sqrt(xseis**2+xggn**2)
	elif "BS" in optic or "SR" in optic:
		return np.sqrt((1.5e-10*f**(-18))**2+(2.8e-14*f**(-7))**2)
	elif "PR" in optic:
		return np.sqrt((2.2e-9*f**(-10))**2+(2.9e-10*f**(-8))**2)

def actuatornoise(optic):
	if "ITM" in optic or "ETM" in optic:
		return f*0
	elif "BS" in optic:
		return np.sqrt((4.6e-13*f**(-8))**2+(1.0e-13*f**(-6))**2+(1.5e-15*f**(-3))**2+(1.1e-17*f**(-1.5))**2)
	elif "SR" in optic:
		return np.sqrt((6.5e-13*f**(-8))**2+(2.0e-13*f**(-6))**2+(4.3e-15*f**(-3))**2+(3.2e-17*f**(-1.5))**2)
	elif "PR" in optic:
		return np.sqrt((1.7e-11*f**(-5))**2+(6.3e-13*f**(-3))**2+(2.2e-14*f**(-2))**2)

def suspensionthermalnoise(optic):
	if "ITM" in optic or "ETM" in optic:
		return CryopayloadSuspensionThermal()
	else:
		return RoomTemperatureSuspensionThermal(optic)

def mirrorthermalnoise(optic):
	return MirrorThermal(optic)

def quantumnoise(dof,RSE="DRSE"):
	if dof=="DARM":
		return DARMQuantumNoise(RSE=RSE)
	elif dof=="MICH":
		return 1.2e-16*np.ones_like(f)
	elif dof=="PRCL":
		return 9.1e-17*np.sqrt(1+(f/3.6e4)**2)
	elif dof=="SRCL":
		return 1.9e-16*np.sqrt(1+(f/4.5e4)**2)
	elif dof=="CARM":
		return np.sqrt((4.6e-21*np.sqrt(1+(f/1.6)**2))**2+(1.2e-15*f**-3)**2)

##### NOISE CALCULATION FUNCTIONS
def CryopayloadSuspensionThermal():
	mass=22.8	   #mirror mass (kg)
	Tm=22		   #temperature at TM (K)

	#####suspension parameters
	rhos=4.0e3	   #density of suspension wire (kg/m**3) #sapphire
	
	#mass (kg)
	m1=20.5		  #IM
	m2=0.055*4	   #blade spring
	m3=mass		  #TM
	
	#resonant frequency of blade spring (Hz)
	wh2=2*pi*2e3	 #horizontal
	fmg=14.5		 #vertical
	
	#radius of wire (m)
	r1=0.6e-3/2	 #wire1 between IM and MN; scale with IM mass
	r2=1.6e-3/2	  #wire2 between blade spring and TM
	
	#Youngs modulus (Pa)
	Y1=1.3e11		#wire1 #CuBe
	Y2=4.0e11		#blade spring #sapphire
	Y3=4.0e11		#wire2 #sapphire
	
	#length of wire (m)
	l1=26.1e-2	   #wire1
	l2=0.5e-2		#blade spring
	l3=35e-2		 #wire2
	
	#loss angle of wire
	phi1=5e-6		#wire1
	phi2=7e-7		#blade spring
	phi3=2e-7		#wire2
	
	#temperature [k]
	T1=16			#wire1
	T2=16			#blade spring
	T3=(T2+Tm)/2	 #wire2
	
	#loss angle normalized by T3
	phih1=phi1*T1/T3 #horizontal wire1
	phih2=phi2*T2/T3 #horizontal blade spring
	phih3=phi3	   #horizontal wire2
	phiv1=phi1*T1/T3 #vertical wire1
	phiv2=phi2*T2/T3 #vertical blade spring
	phiv3=phi3	   #vertical wire2
	
	#coupling from vertical mode to horizontal mode
	VHC=1./200
	
	#number of wires
	N1=4
	N2=4
	N3=4
	
	#number of bending points
	b1=2
	b2=2
	b3=2

	#####suspension thermal noise#####
	#see VIR-015E-09 (2009)
	#violin mode parameters
	lsus=l3
	Ewire=Y3*(1+i*phi3)   #complex Youngs modulus (Pa)
	Icwire=pi*r2**4/4	  #area moment of inertia (m**2)
	tension=mass*g/N3	 #tension per wire (N)
	rholine=rhos*pi*r2**2  #linear density (kg/m)
	
	# wave number of elastic fibers (1/m)
	ks=np.sqrt((-tension+np.sqrt(tension**2+4*Ewire*Icwire*rholine*omega**2))/(2*Ewire*Icwire))
	
	# wave number associated with the flexural stiffness
	ke=np.sqrt((tension+np.sqrt(tension**2+4*Ewire*Icwire*rholine*omega**2))/(2*Ewire*Icwire))
	
	# effective spring constant (N/m)
	# Calculated by K. Somiya and confirmed by K. Komori. The spring constant Eq.(48) in PPPnote (VIR-015E-09) is the limitting value of ke*lsus -> infinity in our case.
	Ksusp=N3*Ewire*Icwire*ke*ks*(ke**2+ks**2)*(ke*np.cos(ks*lsus)*np.sinh(ke*lsus)+ks*np.sin(ks*lsus)*np.cosh(ke*lsus))/(2*ke*ks*(1-np.cos(ks*lsus)*np.cosh(ke*lsus))+(ke**2-ks**2)*np.sin(ks*lsus)*np.sinh(ke*lsus))
	
	#spring constant and resonant frequency
	Ecube=Y1*(1+i*phih1)		  #complex Youngs modulus (Pa)
	I1=pi*r1**4/4				  #area moment of inertia (m**2)
	tensioncube=(m1+m2+m3)*g/N1   #tension per CuBe wire (N)
	
	kh1=N1*tensioncube/(l1-2*np.sqrt(Ecube*I1/tensioncube))	  #hor. spring constatnt of wire1 (N/m)
	kh2=m2*wh2**2*(1+i*phih2)	  #hor. spring constatnt of blade spring (N/m)
	kh3=Ksusp
	
	wv1=np.sqrt(pi*r1**2*Y1*N1/(l1*m1))  #ver. wire1 (Hz)
	kv1=m1*wv1**2*(1+i*phiv1)		 #ver. spring constatnt of wire1 (N/m)
	
	wv2=2*pi*fmg*np.sqrt((m2+m3)/m2)		 #ver. blade spring (Hz)
	kv2=(m2+m3)*(2*pi*fmg)**2*(1+i*phiv2)  #ver. spring constatnt of blade spring (N/m)
	
	wv3=np.sqrt(pi*r2**2*Y3*N3/(l3*m3))  #ver. wire2 (Hz)
	kv3=m3*wv3**2*(1+i*phiv3)		 #ver. spring constatnt of wire2 (N/m)
	
	#matrix of horizontal spring constatnt (N/m)
	Dh11=-m1*omega**2+kh1+kh2
	Dh12=-kh2*np.ones(fres)
	Dh13=0*np.ones(fres)
	Dh21=Dh12
	Dh22=-m2*omega**2+kh2+kh3
	Dh23=-kh3
	Dh31=Dh13
	Dh32=Dh23
	Dh33=-m3*omega**2+kh3
	
	# matrix of vertical spring constatnt (N/m)
	Dv11=-m1*omega**2+kv1+kv2
	Dv12=-kv2*np.ones(fres)
	Dv13=0*np.ones(fres)
	Dv21=Dv12
	Dv22=-m2*omega**2+kv2+kv3
	Dv23=-kv3*np.ones(fres)
	Dv31=Dv13
	Dv32=Dv23
	Dv33=-m3*omega**2+kv3
	
	# for-loop omitted from matlab version for speed up (no change in results)
	#matrix of horizontal impedance (N/m/Hz) and inversion
	Zh=np.array([[Dh11,Dh12,Dh13],[Dh21,Dh22,Dh23],[Dh31,Dh32,Dh33]])/(i*omega)
	invZh=np.transpose(np.linalg.inv(np.transpose(Zh)))	# gives invZh[:,:,k]=np.linalg.inv(Zh[:,:,k])
	#matrix of vertical impedance (N/m/Hz) and inversion
	Zv=np.array([[Dv11,Dv12,Dv13],[Dv21,Dv22,Dv23],[Dv31,Dv32,Dv33]])/(i*omega)
	invZv=np.transpose(np.linalg.inv(np.transpose(Zv)))
	###calculation of eq.(63) in PPPnote (VIR-015E-09)###
	# The fluctuation of TM is derived from 3*3 component of the inverse matrix of impedance.
	Fsqh=4*kb*T3*np.real(invZh)/omega**2
	Sthh=Fsqh[2,2,:]
	Fsqv=4*kb*T3*np.real(invZv)/omega**2
	Sthv=Fsqv[2,2,:]
	
	#strain of thermal noise for 1 TM
	hsusph=np.sqrt(np.abs(Sthh))/L
	hsuspv=np.sqrt(np.abs(Sthv))*VHC/L
	hsusp=np.sqrt(hsusph**2+hsuspv**2)	# hsusp value slightly different from matlab code at 1e-11 level
	
	return hsusp*L

def RoomTemperatureSuspensionThermal(optic):
	# Based on K. Komori's susthermal_PRSR.m and susthermal_BS.m
	T=290		   #temperature at TM (K)

	#####suspension parameters
	if "BS" in optic:
		#mass (kg)
		m1=34.6          #Intermediate mass
		m2=18.71         #Test mass
		m3=22.3          #Recoil mass
		
		#radius of wire (m)
		r1=3.6e-3/2      #wire1 between IM and MN
		r2=0.3e-3/2      #wire2 between IM and TM
		r3=0.65e-3/2     #wire3 between IM and RM
		
		#length of wire (m)
		l1=62.5e-2
		l2=55.1e-2
		l3=55.1e-2
	elif "PR" in optic or "SR" in optic:
		#mass (kg)
		m1=15.6          #Intermediate mass
		m2=10.71         #Test mass
		m3=12.036        #Recoil mass
		
		#radius of wire (m)
		r1=2.5e-3/2      #wire1 between IM and MN
		r2=0.2e-3/2      #wire2 between IM and TM
		r3=0.6e-3/2      #wire3 between IM and RM
		
		#length of wire (m)
		l1=59.85e-2
		l2=58.7e-2
		l3=58.7e-2
	
	m=[m1,m2,m3]
	r=[r1,r2,r3]
	l=[l1,l2,l3]
	
	#Youngs modulus (Pa)
	Y1=1.82e11        #maraging steel (Daido MAS-1)
	Y2=2.0e11        #piano (W. R. Bennett Jr., Science of Musical Sound: Volume 1)
	Y3=4.0e11        #tungsten (http://www.matweb.com/search/DataSheet.aspx?MatGUID=37fde7152eb94e3dbcbde8e2edc0a497)
	Y=[Y1,Y2,Y3]
	
	#density of wire (kg/m^3)
	rho1=8.02e3
	rho2=7.83e3
	rho3=19.3e3
	rho=[rho1,rho2,rho3]
	
	#loss angle of wire
	phi1=2e-4	# https://labcit.ligo.caltech.edu/~janeen/ConceptualDesign1.pdf
	phi2=2e-4	# https://aip.scitation.org/doi/10.1063/1.4928610
	phi3=1.7e-4	# https://www.sciencedirect.com/science/article/pii/S037596019900184X
	phi=[phi1,phi2,phi3]

	#number of wires
	N1=1
	N2=4
	N3=4
	N=[N1,N2,N3]

	#tension per wire (N)
	tension1=(m1+m2+m3)*g/N1
	tension2=m2*g/N2
	tension3=m3*g/N3
	tension=[tension1,tension2,tension3]

	#coupling from vertical mode to horizontal mode
	VHC=1./200

	#####suspension thermal noise#####
	#see VIR-015E-09 (2009)
	#spring constant for horizontal mode including violin
	kh=np.zeros((fres,3))*i
	kv=np.zeros((fres,3))*i
	for j in range(3):
		E=Y[j]*(1+i*phi[j])	#complex Youngs modulus (Pa)
		Ic=pi*r[j]**4/4		#area moment of inertia (m^4)
		rholine=rho[j]*pi*r[j]**2	#linear density (kg/m)
		# wave number of elastic fibers (1/m)
		ks=np.sqrt((-tension[j]+np.sqrt(tension[j]**2+4*E*Ic*rholine*omega**2))/(2*E*Ic))

		# wave number associated with the flexural stiffness
		ke=np.sqrt((tension[j]+np.sqrt(tension[j]**2+4*E*Ic*rholine*omega**2))/(2*E*Ic))

		# effective spring constant (N/m)
		# Calculated by K. Somiya and confirmed by K. Komori. The spring constant Eq.(48) in PPPnote (VIR-015E-09) is the limitting value of ke*lsus -> infinity in our case.
		lsus=l[j]
		#kh[:,j]=N[j]*E*Ic*ke*ks*(ke**2+ks**2)*(ke*np.cos(ks*lsus)*np.sinh(ke*lsus)+ks*np.sin(ks*lsus)*np.cosh(ke*lsus))/(2*ke*ks*(1-np.cos(ks*lsus)*np.cosh(ke*lsus))+(ke**2-ks**2)*np.sin(ks*lsus)*np.sinh(ke*lsus))
		# If we adopt Eq.(48), use the following (To avoid overflow of realmax in matlab, the following had to be used; difference is 1e-6 level in terms of resulting thermal noise)
		kh[:,j]=N[j]*E*Ic*ke*ks*(ke**2+ks**2)*(ke*np.cos(ks*lsus)+ks*np.sin(ks*lsus))/(-2*ke*ks*np.cos(ks*lsus)+(ke**2-ks**2)*np.sin(ks*lsus))

		#vertical spring constant
		kv[:,j]=pi*r[j]**2*E*N[j]/lsus*np.ones(fres)
	
	#matrix of horizontal spring constatnt (N/m)
	Dh11=-m1*omega**2+kh[:,0]+kh[:,1]+kh[:,2]
	Dh12=-kh[:,1]
	Dh13=-kh[:,2]
	Dh21=Dh12
	Dh22=-m2*omega**2+kh[:,1]
	Dh23=0*np.ones(fres)
	Dh31=Dh13
	Dh32=Dh23
	Dh33=-m3*omega**2+kh[:,2]
	
	# matrix of vertical spring constatnt (N/m)
	Dv11=-m1*omega**2+kv[:,0]+kv[:,1]+kv[:,2]
	Dv12=-kv[:,1]
	Dv13=-kv[:,2]
	Dv21=Dv12
	Dv22=-m2*omega**2+kv[:,1]
	Dv23=0*np.ones(fres)
	Dv31=Dv13
	Dv32=Dv23
	Dv33=-m3*omega**2+kv[:,2]
	
	# for-loop omitted from matlab version for speed up (no change in results)
	#matrix of horizontal impedance (N/m/Hz) and inversion
	Zh=np.array([[Dh11,Dh12,Dh13],[Dh21,Dh22,Dh23],[Dh31,Dh32,Dh33]])/(i*omega)
	invZh=np.transpose(np.linalg.inv(np.transpose(Zh)))	# gives invZh[:,:,k]=np.linalg.inv(Zh[:,:,k])
	#matrix of vertical impedance (N/m/Hz) and inversion
	Zv=np.array([[Dv11,Dv12,Dv13],[Dv21,Dv22,Dv23],[Dv31,Dv32,Dv33]])/(i*omega)
	invZv=np.transpose(np.linalg.inv(np.transpose(Zv)))
	###calculation of eq.(63) in PPPnote (VIR-015E-09)###
	# The fluctuation of TM is derived from 2*2 component of the inverse matrix of impedance.
	Fsqh=4*kb*T*np.real(invZh)/omega**2
	Sthh=Fsqh[1,1,:]
	Fsqv=4*kb*T*np.real(invZv)/omega**2
	Sthv=Fsqv[1,1,:]
	
	#strain of thermal noise for 1 TM
	hsusph=np.sqrt(np.abs(Sthh))/L
	hsuspv=np.sqrt(np.abs(Sthv))*VHC/L
	hsusp=np.sqrt(hsusph**2+hsuspv**2)
	
	return hsusp*L

def MirrorThermal(optic):
	#####mirror parameters
	if "ITM" in optic or "ETM" in optic:	# Sapphire
		rhom=4.0e3	  #density of mirror (kg/m**3) #sapphire
		radius=11e-2	#radius of mirror (m)
		height=15e-2	#thickness of mirror (m)
		mass=22.8	   #mirror mass (kg)
		w=3.5e-2	   #beam radius at ETM (m)
		Tm=22		   #temperature at TM [k]
		E0=400e9		#Youngs modulus of mirror (Pa), see Phys. Lett. A 264, 1 (1999)
		sigma=0.29	  #poisson ratio of mirror, see Phys. Lett. A 264, 1 (1999)
		Qm=1e8		  #Q-value of mirror

		#From Uchiyama-san. Origninally from Touloukian and Ho, Thermophysical Properties of Matter: The TRPC Data Series, Plenum Press (1970).
		#see PRL 108, 141101 (2012).
		kappa=(15700./15880)*((5*Tm**2.75)**(-4./5)+(10**10.25*Tm**(-3.8))**(-4./5))**(-5./4) #thermal conductivity
		Cth=(0.69/0.80)*Tm**3.14/3.8/rhom  #specific heat per unit mass
		alpha=(5.6/5.496)*Tm**2.99/10**12.15  #thermal linear expansion
	elif "PR" in optic or "SR" in optic or "BS" in optic:	# Fused silica
		rhom=2.2e3	  #density of mirror (kg/m**3) #fused silica
		if "BS" in optic:
			radius=18.5e-2	#radius of mirror (m)
			height=8e-2	#thickness of mirror (m)
			mass=18.9	   #mirror mass (kg)
		else:
			radius=12.5e-2	#radius of mirror (m)
			height=10e-2	#thickness of mirror (m)
			mass=10.8	   #mirror mass (kg)
		w_list={"BS":3.62e-2,"SRM":0.43e-2,"SR2":0.43e-2,"SR3":3.67e-2,"PRM":0.46e-2,"PR2":0.46e-2,"PR3":3.66e-2}
		w=w_list[optic]	   #beam radius (m)
		Tm=290		   #temperature at TM [k]
		E0=73.2e9		#Youngs modulus of mirror (Pa); silica, see PRD 93, 012007 (2016)
		sigma=0.164	  #poisson ratio of mirror; silica, see PRD 93, 012007 (2016)
		Qm=1/(6.5e-12/height+7.6e-12*f**0.77)		  #Q-value of mirror, see Physics Letters A 352, 3 (2006)

		#Based on Somiya's Mathematica code
		kappa=1.38 #thermal conductivity	# NEEDS CONFIRMATION
		Cth=1.64e6/rhom  #specific heat per unit mass	# NEEDS CONFIRMATION
		alpha=5.1e-7  #thermal linear expansion	# NEEDS CONFIRMATION

	#####coating parameters
	#number of coating layers
	if "ITM" in optic:
		Ncoa=11	   #ITM
	elif "ETM" in optic:
		Ncoa=20	   #ETM
	else:	# for fused silica mirrors
		Ncoa_list={"BS":1,"SRM":4,"SR2":18,"SR3":18,"PRM":4,"PR2":18,"PR3":18}	# Based on Somiya's calculation from reflectivity
		Ncoa=Ncoa_list[optic]
	
	#Youngs modulus (Pa)
	Ys=73.2e9	   #silica, see PRD 93, 012007 (2016)
	Yt=140e9		#tantala, see PRD 70, 082003 (2004)
	
	#poisson ratio
	sigmas=0.164	#silica, see PRD 93, 012007 (2016)
	sigmat=0.23	 #tantala, see PRD 70, 082003 (2004)
	
	#refractive index
	n1=1.442		#silica, see PRD 93, 012007 (2016)
	n2=2.035		#tantala, see CQG 27, 084030 (2010)
	
	#thickness of layers (m)
	dcoas=(Ncoa+1)*lamb/(4*n1)  #silica
	dcoat=Ncoa*lamb/(4*n2)	  #tantala
	if optic=="BS": # tantala/silica/tantala for BS, see https://gwdoc.icrr.u-tokyo.ac.jp/cgi-bin/private/DocDB/ShowDocument?docid=3347
		dcoas=Ncoa*lamb/(4*n1)  #silica
		dcoat=(Ncoa+1)*lamb/(4*n2)	  #tantala
	
	#loss angle
	phic1=3e-4	   #silica coating
	phic2=5e-4	   #tantala coating

	######mirror thermal noise#####
	#strain of substrate thermal noise for 1 TM
	#see CQG, 29, 124007 (2012)
	hmirsub=1/L*np.sqrt(4*kb*Tm*(1-sigma**2)/(np.sqrt(pi)*E0*Qm*omega)*(1/w))
	
	#thermoelastic noise calculations
	#see PRD, 82, 127101 (2010)
	#normalized frequency
	omegac=omega*Cth*rhom*w**2/(2*kappa)
	
	#calculation of Jhyper
	#It is confirmed by K. Komori that this integration is the same as the analytical calculation using error function.
	resolution=0.01
	up=10	#upper limit of integration instead of infinity
	bin=int(floor(up/resolution))
	u=np.linspace(0,up,bin+1)
	
	# for-loop omitted and simplified from matlab code for speed up (no change in result)
	uu,omec=np.meshgrid(u,omegac)
	
	integ=u**2*np.exp(-u**2/2)*(1-u/np.sqrt(2)*np.sqrt((u**2+np.sqrt(u**4+omec**2))/(u**4+omec**2)))
	
	integral=np.sum(integ[:,1:]+integ[:,:-1],axis=1)/2*resolution # nulling changed from matlab code
	
	Jhyperint=np.sqrt(2/pi)*integral/omegac**2
	
	#strain of thermoelastic noise for 1 TM
	hmirelaint=1/L*np.sqrt(4*kb*Tm**2*alpha**2*(1+sigma)**2/(np.sqrt(pi)*kappa)*(w*Jhyperint))
	
	#coating thermal noise calculations
	#see CQG, 19, 897 (2002)
	#thermal noise of silica coating
	hmircoas=np.sqrt(4*kb*Tm/(pi*E0**2*omega))*np.sqrt(phic1*(Ys**2*(1+sigma)**2*(1-2*sigma)**2+E0**2*(1+sigmas)**2*(1-2*sigmas))/(Ys*(1-sigmas**2))*(dcoas/w**2))
	
	#thermal noise of tantala coating
	hmircoat=np.sqrt(4*kb*Tm/(pi*E0**2*omega))*np.sqrt(phic2*(Yt**2*(1+sigma)**2*(1-2*sigma)**2+E0**2*(1+sigmat)**2*(1-2*sigmat))/(Yt*(1-sigmat**2))*(dcoat/w**2))
	
	#strain of coating thermal noise for 1 TM
	hmircoa=1/L*np.sqrt(hmircoas**2+hmircoat**2)
	
	#total mirror thermal noise for 1 TM
	hmir=np.sqrt(hmirsub**2+hmirelaint**2+hmircoa**2)
	
	return hmir*L

def DARMQuantumNoise(RSE="DRSE"):
	#####interferometer parameters
	if RSE=='DRSE':	 #detuned RSE
		phidet=pi/180*86.5   #detune phase
	  	xi=pi/180*135.1	  #homodyne phase
	elif RSE=='BRSEhomo':	#broadband RSE with homodyne
		phidet=pi/180*90
	 	xi=pi/180*119.1
	elif RSE=='BRSE':	#broadband RSE without homodyne
	  	phidet=pi/180*90
	  	xi=pi/180*90

	trans=0.004		  #power transmittivity of ITM
	rSRM=0.92			#amplitude reflectivity of SRM
	roundtriploss=100e-6 #round trip loss of arm
	lambSR=0.002	   #power loss at SRM
	lambPD=0.1		 #power loss at PD
	Sadd=0*(pi**2/8-1)
	I0=674.	#power at BS
	
	mass=22.8	   #mirror mass (kg)
	
	######quantum noise#####
	#see PRD, 64, 042006 (2001)
	#parameters
	gamma=trans*c/(4*L)	   #cavity pole (Hz)
	tSRM=np.sqrt(1-rSRM**2)	   #SRM transmittance
	beta=np.arctan(omega/gamma)   #phase shift in the arm cavity
	
	#laser power quantum noise reaches the SQL at the cavity pole
	ISQL=mass*L**2*gamma**4/(4*omegal)
	
	#coefficient showing how much the mirror moves
	kappa=I0/ISQL*2*gamma**4/(omega**2*(gamma**2+omega**2))
	
	#loss parameter of arms defined in KLMTV paper
	epsilon=2*roundtriploss/trans
	
	#quantum noise calculations
	# strain of SQL
	hsql=np.sqrt(8*hbar/(mass*omega**2*L**2))
	
	#components of eq.(5.13) in PRD, 64, 042006 (2001)
	C11=np.sqrt(1-lambPD)*((1+rSRM**2)*(np.cos(2*phidet)+kappa*np.sin(2*phidet)/2)-2*rSRM*np.cos(2*beta)-epsilon/4*(-2*(1+np.exp(2*i*beta))**2*rSRM+4*(1+rSRM**2)*np.cos(beta)**2*np.cos(2*phidet)+(3+np.exp(2*i*beta))*kappa*(1+rSRM**2)*np.sin(2*phidet))+lambSR*(np.exp(2*i*beta)*rSRM-(1+rSRM**2)/2*(np.cos(2*phidet)+kappa*np.sin(2*phidet)/2)))
	C22=C11
	C12=np.sqrt(1-lambPD)*tSRM**2*(-(np.sin(2*phidet)+kappa*np.sin(phidet)**2)+epsilon*np.sin(phidet)/2*((3+np.exp(2*i*beta))*kappa*np.sin(phidet)+4*np.cos(beta)**2*np.cos(phidet))+lambSR/2*(np.sin(2*phidet)+kappa*np.sin(phidet)**2))
	C21=np.sqrt(1-lambPD)*tSRM**2*((np.sin(2*phidet)-kappa*np.cos(phidet)**2)+epsilon*np.cos(phidet)/2*((3+np.exp(2*i*beta))*kappa*np.cos(phidet)-4*np.cos(beta)**2*np.sin(phidet))+lambSR/2*(-np.sin(2*phidet)+kappa*np.cos(phidet)**2))
	D1sig=np.sqrt(1-lambPD)*(-(1+rSRM*np.exp(2*i*beta))*np.sin(phidet)+epsilon/4*(3+rSRM+2*rSRM*np.exp(4*i*beta)+np.exp(2*i*beta)*(1+5*rSRM))*np.sin(phidet)+lambSR/2*np.exp(2*i*beta)*rSRM*np.sin(phidet))
	D2sig=np.sqrt(1-lambPD)*(-(-1+rSRM*np.exp(2*i*beta))*np.cos(phidet)+epsilon/4*(-3+rSRM+2*rSRM*np.exp(4*i*beta)+np.exp(2*i*beta)*(-1+5*rSRM))*np.cos(phidet)+lambSR/2*np.exp(2*i*beta)*rSRM*np.cos(phidet))
	P11=np.sqrt(1-lambPD)/2*np.sqrt(lambSR)*tSRM*(-2*rSRM*np.exp(2*i*beta)+2*np.cos(2*phidet)+kappa*np.sin(2*phidet))
	P22=P11
	P12=-np.sqrt(1-lambPD)*np.sqrt(lambSR)*tSRM*np.sin(phidet)*(2*np.cos(phidet)+kappa*np.sin(phidet))
	P21=np.sqrt(1-lambPD)*np.sqrt(lambSR)*tSRM*np.cos(phidet)*(2*np.sin(phidet)-kappa*np.cos(phidet))
	Q11=np.sqrt(lambPD)*(np.exp(-2*i*beta)+rSRM**2*np.exp(2*i*beta)-rSRM*(2*np.cos(2*phidet)+kappa*np.sin(2*phidet))+epsilon/2*rSRM*(np.exp(-2*i*beta)*np.cos(2*phidet)+np.exp(2*i*beta)*(-2*rSRM-2*rSRM*np.cos(2*beta)+np.cos(2*phidet)+kappa*np.sin(2*phidet))+2*np.cos(2*phidet)+3*kappa*np.sin(2*phidet))-lambSR/2*rSRM*(2*rSRM*np.exp(2*i*beta)-2*np.cos(2*phidet)-kappa*np.sin(2*phidet)))
	Q22=Q11
	Q12=0
	Q21=0
	N11=np.sqrt(1-lambPD)*np.sqrt(epsilon/2)*tSRM*(kappa*(1+rSRM*np.exp(2*i*beta))*np.sin(phidet)+2*np.cos(beta)*(np.exp(-i*beta)*np.cos(phidet)-rSRM*np.exp(i*beta)*(np.cos(phidet)+kappa*np.sin(phidet))))
	N22=-np.sqrt(1-lambPD)*np.sqrt(2*epsilon)*tSRM*(-np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.cos(phidet)
	N12=-np.sqrt(1-lambPD)*np.sqrt(2*epsilon)*tSRM*(np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.sin(phidet)
	N21=np.sqrt(1-lambPD)*np.sqrt(epsilon/2)*tSRM*(-kappa*(1+rSRM)*np.cos(phidet)+2*np.cos(beta)*(np.exp(-i*beta)+rSRM*np.exp(i*beta))*np.cos(beta)*np.sin(phidet))
	M=0
	
	#power of shot noise
	Ssh=hsql**2/(2*tSRM**2*kappa*np.abs(D1sig*np.sin(xi)+D2sig*np.cos(xi))**2)*(np.abs(C11*np.sin(xi)+C21*np.cos(xi))**2+np.abs(C12*np.sin(xi)+C22*np.cos(xi))**2+np.abs(P11*np.sin(xi)+P21*np.cos(xi))**2+np.abs(P12*np.sin(xi)+P22*np.cos(xi))**2+np.abs(Q11*np.sin(xi)+Q21*np.cos(xi))**2+np.abs(Q12*np.sin(xi)+Q22*np.cos(xi))**2+np.abs(N11*np.sin(xi)+N21*np.cos(xi))**2+np.abs(N12*np.sin(xi)+N22*np.cos(xi))**2+(np.abs(M)*np.sqrt(Sadd))**2)
	
	#These calculations are consistence with the result from Haino.
	
	#strain of shot noise without squeezing
	hsh=np.sqrt(Ssh)
	
	return hsh*L

##### CLASSES
class OpticDisplacementSensitivity():
	def __init__(self,op):
		self.optic=op
		self.freq=f
		self.seis=seismicnoise(op)
		self.susp=suspensionthermalnoise(op)
		self.mirr=mirrorthermalnoise(op)
		self.act=actuatornoise(op)
		self.total=np.sqrt(self.seis**2+self.susp**2+self.mirr**2+self.act**2)

class DofDisplacementSensitivity():
	def __init__(self,dof,optic_list,weight_list,RSE="DRSE"):
		self.dof=dof
		self.freq=f
		OpticDict={}
		for op in optic_list:
			OpticDict[op]=OpticDisplacementSensitivity(op)
		for kk in ["seis","susp","mirr","act"]:
			n=0
			ii=0
			for op in optic_list:
				n+=(weight_list[ii]*getattr(OpticDict[op],kk))**2
				ii+=1
			setattr(self,kk,np.sqrt(n))
		self.quant=quantumnoise(dof,RSE=RSE)
		self.total=np.sqrt(self.seis**2+self.susp**2+self.mirr**2+self.quant**2+self.act**2)